/* wavframes.c - input of waveform and division to frames */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "wavframes.h"
#include "config.h"

/* just open the file */
void  open_raw_file (char *namein) {
  filein = fopen(namein, "rb");
  if (filein == NULL) {
    fprintf (stderr, "problem opening input wav file `%s'\n", namein); 
    exit (1); 
  }
}

/* just open the file */
void  open_wav_file (char *namein) {
//  unsigned int	BitsPerSample;
  unsigned long  RC,DInfo;
  TFmtChunk  Fmt;
  memchr(&Fmt, 0, sizeof(Fmt)); // warning: statement with no effect

  filein = fopen(namein, "rb"); //open file
  if (filein == NULL) {
    fprintf (stderr, "problem opening input wav file `%s'\n", namein); 
    exit (1); 
  }

  fseek( filein, 0, SEEK_SET);

  if (!FindChunk("RIFF"))
    { 	/* Invalid file format */
      fclose(filein);
      fprintf (stderr, "Invalid file format. Not a WAV file.\n"); 
      exit (1); 
    }
  fread(&RC, sizeof(RC), 1, filein);
  
  if (!FindChunk("fmt "))  /* This chunk is in a Wave chunk */
    {
      fclose(filein);
      fprintf (stderr, "Invalid file format. Not a WAV file.\n"); 
      exit (1); 
    }
    fread(&Fmt, sizeof(Fmt), 1, filein);

    if (!FindChunk("data "))
    {
      fclose(filein);
      fprintf (stderr, "Invalid file format. DATA chunk not found.\n"); 
      exit (1); 
    }

    /* supported format is WAVE_FORMAT_PCM - uncompressed sampled sound  */
	if(Fmt.wFormatTag != 1)
    {
      fclose(filein);
      fprintf (stderr, "Unsupported file format. Supported format is WAVE_FORMAT_PCM - uncompressed sampled sound\n"); 
      exit (1); 
    }

    /* I can read only 1channel files */
    if( Fmt.nChannels != 1 )
    {
      fclose(filein);
      fprintf (stderr, "Unsupported file format. Supported only 1 channel files.\n"); 
      exit (1); 
    }

    /* I can read 16 bits per sample */
    //    nBitsPerSample = Fmt.nBlockAlign * 8 / Fmt.nChannels;
    if(Fmt.nBlockAlign * 8 / Fmt.nChannels != 16 )
    {
      fclose(filein);
      fprintf (stderr, "Unsupported file format. Supported only 16bits per sample.\n"); 
      exit (1); 
    }

    /* I can read only files with 8kHz sampling */
    if( Fmt.nSamplePerSec != 8000 )
    {
      fclose(filein);
      fprintf (stderr, "Unsupported file format. Supported sampling 8kHz.\n"); 
      exit (1); 
    }
    fread(&DInfo, sizeof(DInfo), 1, filein);
    //    nSampleFrequency = Fmt.nSamplePerSec;
    //    nDataSize = DInfo / Fmt.nBlockAlign;
}

// This function find chunk next in the wave file and move the file pointer
// next to chunk label
// chunk name is four bytes long
// 1 is OK
// 0 chunk not found
int FindChunk(char *name)
{
    char 	buff[4];
    int 	i;

    // Fill the circular buffer for first step
    for(i = 0; i < 4; ++i)
    	{
            if(feof(filein))
            	return(0);
			buff[i] = (char)fgetc(filein);
        }

   	if(memcmp(name, buff, 4) == 0)
        return 1; // chunk found
   	if(memcmp(buff, "data", 4) == 0)
    	return 0; // for speed, data chunk is in the end

    // Here I find chunk name next in the file
    while(!feof(filein))
    {
    	buff[0] = buff[1];
        buff[1] = buff[2];
        buff[2] = buff[3];
        buff[3] = (char)fgetc(filein);
    	if(memcmp(name, buff, 4) == 0)
            return 1; // chunk found
	   	if(memcmp(buff, "data", 4) == 0)
    		return 0; // for speed, data chunk is in the end
    }
    return (0);  // chunk is not in the file
};

/* allocate for speech frame - floats */
float * allocate_speech_frame (void) {
  void *ptr;
  float *f; 

  ptr = malloc (WIN_LEN * sizeof(float)); 
  if (ptr == NULL) {
    fprintf (stderr, "Bad malloc\n"); 
    exit (1); 
  }
  f = (float*)ptr; 
  return (f);
}

/* get new speech frame: 0 is returned if not full frame read, 1 if OK */
int new_frame (float * frame) {
  static int first_frame=1; 
  static short *aux_inframe; 
  short *saux; 
  int samples_read,ii; 
  void *ptr; 
  float *faux; 

  if (first_frame) {
    /* allocate short samples to be read from file */ 
    ptr = malloc (WIN_LEN * sizeof(short)); 
    if (ptr == NULL) {
      fprintf (stderr, "Bad malloc\n"); 
      exit (1); 
    }
    aux_inframe = (short *)ptr;
    
    /* read full frame of samples */
    samples_read = fread (aux_inframe, sizeof(short), WIN_LEN, filein);

    /* if not enough, just return 0 */
    if (samples_read < WIN_LEN) 
      return (0);
    
    /* ... and convert */
    saux = aux_inframe; faux = frame;
    for (ii=0; ii<WIN_LEN; ii++) 
      *(faux++) = ((float)(*(saux++))) / SHORT2FLOAT; 

    /* 1st frame will never come again ... */
    first_frame = 0; 
    return (1); 
  }
  
  /* NORMAL CASE - read just new  samples */
  samples_read = fread (aux_inframe, sizeof(short), WIN_SHIFT, filein);
  
  /* if not enough, just return 0 */
  if (samples_read < WIN_SHIFT) 
    return (0);

  /* so we have enough - shift old samples to the beginning */
  memmove (frame, frame + WIN_SHIFT, (WIN_LEN-WIN_SHIFT) * sizeof(float));
  
  /* ... and convert new samples */
  saux = aux_inframe; 
  faux = frame + WIN_LEN-WIN_SHIFT; 
  for (ii=0; ii<WIN_SHIFT; ii++) 
    *(faux++) = ((float)(*(saux++))) / SHORT2FLOAT; 
  
  return (1);
}

/* cloding the wav file ... really simple  */
void close_wav_file (void) {
  fclose (filein);
}
