/* zakladni soft pro zpracovani signalu - wire. Nacte rec, rozsegmentuje 
   ji do ramcu a ramce narve do vystupniho bin. souboru jako floats 

   volani je velmi jednoduche: 

   wire infile outfile

   infile obsahuje waveform, outfile matici jednotlivych ramcu. 
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "wavframes.h"   /* waveform input and framing */
#include "binout.h"      /* bin file out */
#include "speech.h"      /* speech processing */
#include "config.h"      /* all config parameters */ 

int main (int argc, char **argv) {
  char *namein;
  char *nameout; 
//  int res; 
  float *frame;   /* speech frame - already floats */ 
  float *params;  /* out parameters - here just a copy of samples */
  int frmcnt=0; 
  char *extension; 

  if (argc != 3) {
    fprintf (stderr, "Usage: wire infile(raw or WAV) outfile\n");
    exit (1); 
  }
  namein = argv[1]; 
  nameout = argv[2]; 

  /* the opening function is different for RAW and WAV ... */
  extension = namein + strlen(namein) - 3; /* the extension has 3 chars */
//  if (!strcasecmp (extension, "wav")) { // implicit declaration of function `strcasecmp'
  if (!strcmp (extension, "wav")) {

    fprintf (stderr, "WAV FILE: "); 
    open_wav_file (namein); 
  }
  else {
    fprintf (stderr, "RAW FILE: "); 
    open_raw_file (namein);
  } 
  frame = allocate_speech_frame (); 

  open_bin_file (nameout); 
  params = allocate_out_params (); 

  /* process frames */ 
  while (new_frame(frame)) {
    process_frame (frame, params); 
    write_params (params); 
    frmcnt ++; 
  }
  
  close_wav_file(); 
  close_bin_file();
  free(frame);
  free(params); 

  fprintf (stderr, "%s -> %s %d frames\n",namein,nameout,frmcnt); 
}



